/*
******************************************************************************
*  Name     : BitmapUtilsDemoEng.cpp
*  Date     : 30.7.04
*  Description:
*     A simple drawing engine, using DirectScreenAccess. 
*     
*  Copyright: Susi Games, Germany
******************************************************************************
*/

#include "BitmapUtilsDemoEng.h"

#include <eikenv.h>
#include "CopyUtils.h"



_LIT(KBitmapUtilsDemoTxt, "DirectScreenAccessDemo");

CBitmapUtilsDemoEng::CBitmapUtilsDemoEng(RWsSession& aClient, CWsScreenDevice& aScreenDevice, RWindow& aWindow)
: CTimer(CActive::EPriorityStandard),
	iClient(aClient),
	iScreenDevice(aScreenDevice),
	iWindow(aWindow),
    iDirectScreenAccess(0),
    iGc(0),
    iRegion(0),	
    iPosition(0,0),
    iDrawing(EFalse),
    iScreenAddr(0),
    iOffScreenBmp(0)  
{    
    iDrawRect.SetRect(iPosition, aWindow.Size());
    // calculate the (used) frame buffer size in bytes
    iFrameBufLen = iDrawRect.Width() * iDrawRect.Height() * 2;
	iWidth = 176;
	iHeight = 208;
	iLoopInterval = 18000;

	// maximum speed on slow emulator
#ifdef __WINS__
	iLoopInterval = 10000;
#endif

}


CBitmapUtilsDemoEng* CBitmapUtilsDemoEng::NewL(RWsSession& aClient, CWsScreenDevice& aScreenDevice, RWindow& aWindow)
{
    CBitmapUtilsDemoEng* self = new (ELeave) CBitmapUtilsDemoEng(aClient, aScreenDevice, aWindow);
    CleanupStack::PushL(self);
    self->ConstructL();
    CleanupStack::Pop(); // self;
    return self;
}


CBitmapUtilsDemoEng::~CBitmapUtilsDemoEng()
{
    if(IsActive())
	    Cancel();

	delete iDirectScreenAccess;	
    delete iOffScreenBmp;    
}


void CBitmapUtilsDemoEng::ConstructL()
{
	// contruct the timer
	CTimer::ConstructL();

	// Create the DSA object
	iDirectScreenAccess = CDirectScreenAccess::NewL(iClient, iScreenDevice, iWindow, *this);
	CActiveScheduler::Add(this);

    // create the offscreen bitmap with no alpha information
    iOffScreenBmp = new (ELeave) CFastBitmap(1);
	iOffScreenBmp->Create(iScreenDevice.SizeInPixels(), EColor4K);

	iUseFrameBuffer = ETrue;
	
// for emulator, always use offscreen bitmap
#if defined(__WINS__)
    iUseFrameBuffer = EFalse;
#endif
    
    if(iUseFrameBuffer)        
    {
        // get screen buffer address
        TScreenInfoV01 screenInfo;
	    TPckg<TScreenInfoV01> sInfo(screenInfo);
	    UserSvr::ScreenInfo(sInfo);
         
        iScreenAddr = screenInfo.iScreenAddressValid ? (TUint8*)screenInfo.iScreenAddress : 0;
        User::LeaveIfNull(iScreenAddr);

        // skip the palette data in the beginning of frame buffer (16 entries in 12bit mode)
        iScreenAddr += 16 * 2;

        // initialise the raw redraw event (used when drawing directly to frame buffer)
        iRedraw.Set(TRawEvent::ERedraw);
    }    
}


void CBitmapUtilsDemoEng::StartDrawingL()
{	
	if (iDrawing)
		User::Panic(KBitmapUtilsDemoTxt, KErrInUse);
    
    // Initialise DSA

    // Trap the call to CDirectScreenAccess->StartL() to suppress system
    // error notes (e.g. active screen saver may cause problems)
	TRAPD(dsaErr, iDirectScreenAccess->StartL());
    if(dsaErr == KErrNone)
    {

	    // Get graphics context for it
	    iGc = iDirectScreenAccess->Gc();

	    // Get region that DSA can draw in
	    iRegion = iDirectScreenAccess->DrawingRegion();

	    // Set the display to clip to this region
	    iGc->SetClippingRegion(iRegion);

        iDrawing = ETrue;

        // request a timer event after a defined interval
        After(TTimeIntervalMicroSeconds32(iLoopInterval));
    } else
	{
		// error
	}
}


void CBitmapUtilsDemoEng::StopDrawing()
{
	if (!iDrawing)
		User::Panic(KBitmapUtilsDemoTxt, KErrGeneral);
	
	// Cancel timer and display
    // This is CActive::Cancel, which calls derived method DoCancel
	Cancel();
	iDrawing = EFalse;
}
	

void CBitmapUtilsDemoEng::Restart(RDirectScreenAccess::TTerminationReasons /*aReason*/)
{
	// Restart display
	// Note that this will result in the clipping region being updated
	// so that menus, overlaying dialogs, etc. will not be drawn over      
	StartDrawingL();    
}


void CBitmapUtilsDemoEng::AbortNow(RDirectScreenAccess::TTerminationReasons /*aReason*/)
{
	// Cancel timer and display
	Cancel();
	iDrawing = EFalse;
}


void CBitmapUtilsDemoEng::RunL()
{

	UpdatePositions();

	// draw background
	if (iBackground!=NULL)
	{
		// no alpha information -> use the (fastest) BitBlt_4K funktion to copy the background
		CCopyUtils::BitBlt_4K(iBackground,iOffScreenBmp,TPoint(0,0));
	}


	// draw two items
	if (iBmp1!=NULL)
	{
		// this bitmap has only mask information -> so use the BitBltMasked function
		CCopyUtils::BitBltMasked(iBmp1,iOffScreenBmp,iPos1);
	}
	if (iBmp2!=NULL)
	{
		// this bitmap has alpha information-> use the (slowest) BitBltMasked_4AlphaValues function
		CCopyUtils::BitBltMasked_4AlphaValues(iBmp2,iOffScreenBmp,iPos2);
	}


    // blit the offscreen bitmap (if used) to screen
    if(!iUseFrameBuffer)
    {        
        iGc->BitBlt(iPosition, iOffScreenBmp, iDrawRect);
        // Force screen update
        iDirectScreenAccess->ScreenDevice()->Update();
    }
    else
    {        
        // copy bitmap contents to frame buffer
        Mem::Copy(iScreenAddr, iOffScreenBmp->DataAddress(), iFrameBufLen);

        // Drawing to screen buffer (ScreenInfo) - force update
        // We do not use Graphics context, so we generate an event to get screen refreshed
        // As a side-effect, the backlight stays on        
        UserSvr::AddEvent(iRedraw);
    }        

	// Renew request
	After(TTimeIntervalMicroSeconds32(iLoopInterval));

}


void CBitmapUtilsDemoEng::DoCancel()
{
	// Cancel timer
	CTimer::DoCancel();

	// Cancel DSA
	iDirectScreenAccess->Cancel();    
}


void CBitmapUtilsDemoEng::SetBackgroundImage(CFastBitmap *aImage)
{
	iBackground = aImage;
}



void CBitmapUtilsDemoEng::SetForegroundBitmaps(CFastBitmap *aBmp1, CFastBitmap *aBmp2)
{
	iBmp1 = aBmp1;
	iBmp2 = aBmp2;
	iPos1.iX = 1;
	iPos1.iY = 1;
	iStep1_x = 1;
	iStep1_y = 2;
	iPos2.iX = 100;
	iPos2.iY = 100;
	iStep2_x = -3;
	iStep2_y = 1;
}


void CBitmapUtilsDemoEng::UpdatePositions()
{
	if (iPos1.iX>=iWidth-iBmp1->SizeInPixels().iWidth || iPos1.iX<=0) iStep1_x=-iStep1_x;
	if (iPos1.iY>=iHeight-iBmp1->SizeInPixels().iHeight || iPos1.iY<=0) iStep1_y=-iStep1_y;
	iPos1.iX += iStep1_x;
	iPos1.iY += iStep1_y;
	if (iPos2.iX>=iWidth-iBmp2->SizeInPixels().iWidth || iPos2.iX<=0) iStep2_x=-iStep2_x;
	if (iPos2.iY>=iHeight-iBmp2->SizeInPixels().iHeight || iPos2.iY<=0) iStep2_y=-iStep2_y;
	iPos2.iX += iStep2_x;
	iPos2.iY += iStep2_y;
}



void CBitmapUtilsDemoEng::ChangeLoopInterval(TInt aAdd)
{
	iLoopInterval += aAdd;
	if (iLoopInterval < 0) iLoopInterval = 0;
}

// End of File
